# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..misc import AddCSVColumn


def test_AddCSVColumn_inputs():
    input_map = dict(
        extra_column_heading=dict(),
        extra_field=dict(),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        out_file=dict(
            extensions=None,
            usedefault=True,
        ),
    )
    inputs = AddCSVColumn.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AddCSVColumn_outputs():
    output_map = dict(csv_file=dict(extensions=None, ), )
    outputs = AddCSVColumn.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from .config import NUMPY_MMAP
from .onetime import OneTimeProperty, setattr_on_read
from .tmpdirs import TemporaryDirectory, InTemporaryDirectory



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from io import open

import os.path as op
import nibabel as nb
import nibabel.trackvis as trk
import numpy as np
from nibabel.trackvis import HeaderError
from nibabel.volumeutils import native_code
from nibabel.orientations import aff2axcodes

from ... import logging
from ...utils.filemanip import split_filename
from ..base import TraitedSpec, File, isdefined
from ..dipy.base import DipyBaseInterface, HAVE_DIPY as have_dipy

iflogger = logging.getLogger('nipype.interface')


def get_vox_dims(volume):
    import nibabel as nb
    from nipype.utils import NUMPY_MMAP
    if isinstance(volume, list):
        volume = volume[0]
    nii = nb.load(volume, mmap=NUMPY_MMAP)
    hdr = nii.header
    voxdims = hdr.get_zooms()
    return [float(voxdims[0]), float(voxdims[1]), float(voxdims[2])]


def get_data_dims(volume):
    import nibabel as nb
    from nipype.utils import NUMPY_MMAP
    if isinstance(volume, list):
        volume = volume[0]
    nii = nb.load(volume, mmap=NUMPY_MMAP)
    hdr = nii.header
    datadims = hdr.get_data_shape()
    return [int(datadims[0]), int(datadims[1]), int(datadims[2])]


def transform_to_affine(streams, header, affine):
    from dipy.tracking.utils import move_streamlines
    rotation, scale = np.linalg.qr(affine)
    streams = move_streamlines(streams, rotation)
    scale[0:3, 0:3] = np.dot(scale[0:3, 0:3],
                             np.diag(1. / header['voxel_size']))
    scale[0:3, 3] = abs(scale[0:3, 3])
    streams = move_streamlines(streams, scale)
    return streams


def read_mrtrix_tracks(in_file, as_generator=True):
    header = read_mrtrix_header(in_file)
    streamlines = read_mrtrix_streamlines(in_file, header, as_generator)
    return header, streamlines


def read_mrtrix_header(in_file):
    fileobj = open(in_file, 'rb')
    header = {}
    iflogger.info('Reading header data...')
    for line in fileobj:
        line = line.decode()
        if line == 'END\n':
            iflogger.info('Reached the end of the header!')
            break
        elif ': ' in line:
            line = line.replace('\n', '')
            line = line.replace("'", "")
            key = line.split(': ')[0]
            value = line.split(': ')[1]
            header[key] = value
            iflogger.info('...adding "%s" to header for key "%s"', value, key)
    fileobj.close()
    header['count'] = int(header['count'].replace('\n', ''))
    header['offset'] = int(header['file'].replace('.', ''))
    return header


def read_mrtrix_streamlines(in_file, header, as_generator=True):
    offset = header['offset']
    stream_count = header['count']
    fileobj = open(in_file, 'rb')
    fileobj.seek(offset)
    endianness = native_code
    f4dt = np.dtype(endianness + 'f4')
    pt_cols = 3
    bytesize = pt_cols * 4

    def points_per_track(offset):
        track_points = []
        iflogger.info('Identifying the number of points per tract...')
        all_str = fileobj.read()
        num_triplets = int(len(all_str) / bytesize)
        pts = np.ndarray(
            shape=(num_triplets, pt_cols), dtype='f4', buffer=all_str)
        nonfinite_list = np.where(np.invert(np.isfinite(pts[:, 2])))
        nonfinite_list = list(nonfinite_list[0])[
            0:-1]  # Converts numpy array to list, removes the last value
        for idx, value in enumerate(nonfinite_list):
            if idx == 0:
                track_points.append(nonfinite_list[idx])
            else:
                track_points.append(
                    nonfinite_list[idx] - nonfinite_list[idx - 1] - 1)
        return track_points, nonfinite_list

    def track_gen(track_points):
        n_streams = 0
        iflogger.info('Reading tracks...')
        while True:
            try:
                n_pts = track_points[n_streams]
            except IndexError:
                break
            pts_str = fileobj.read(n_pts * bytesize)
            nan_str = fileobj.read(bytesize)
            if len(pts_str) < (n_pts * bytesize):
                if not n_streams == stream_count:
                    raise HeaderError('Expecting %s points, found only %s' %
                                      (stream_count, n_streams))
                    iflogger.error('Expecting %s points, found only %s',
                                   stream_count, n_streams)
                break
            pts = np.ndarray(
                shape=(n_pts, pt_cols), dtype=f4dt, buffer=pts_str)
            nan_pt = np.ndarray(shape=(1, pt_cols), dtype=f4dt, buffer=nan_str)
            if np.isfinite(nan_pt[0][0]):
                raise ValueError
                break
            xyz = pts[:, :3]
            yield xyz
            n_streams += 1
            if n_streams == stream_count:
                iflogger.info('100%% : %i tracks read', n_streams)
                raise StopIteration
            try:
                if n_streams % int(stream_count / 100) == 0:
                    percent = int(float(n_streams) / float(stream_count) * 100)
                    iflogger.info('%i%% : %i tracks read', percent, n_streams)
            except ZeroDivisionError:
                iflogger.info('%i stream read out of %i', n_streams,
                              stream_count)

    track_points, nonfinite_list = points_per_track(offset)
    fileobj.seek(offset)
    streamlines = track_gen(track_points)
    if not as_generator:
        streamlines = list(streamlines)
    return streamlines


class MRTrix2TrackVisInputSpec(TraitedSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        desc='The input file for the tracks in MRTrix (.tck) format')
    image_file = File(
        exists=True, desc='The image the tracks were generated from')
    matrix_file = File(
        exists=True,
        desc=
        'A transformation matrix to apply to the tracts after they have been generated (from FLIRT - affine transformation from image_file to registration_image_file)'
    )
    registration_image_file = File(
        exists=True,
        desc='The final image the tracks should be registered to.')
    out_filename = File(
        'converted.trk',
        genfile=True,
        usedefault=True,
        desc='The output filename for the tracks in TrackVis (.trk) format')


class MRTrix2TrackVisOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class MRTrix2TrackVis(DipyBaseInterface):
    """
    Converts MRtrix (.tck) tract files into TrackVis (.trk) format
    using functions from dipy
    Example
    -------
    >>> import nipype.interfaces.mrtrix as mrt
    >>> tck2trk = mrt.MRTrix2TrackVis()
    >>> tck2trk.inputs.in_file = 'dwi_CSD_tracked.tck'
    >>> tck2trk.inputs.image_file = 'diffusion.nii'
    >>> tck2trk.run()                                   # doctest: +SKIP
    """
    input_spec = MRTrix2TrackVisInputSpec
    output_spec = MRTrix2TrackVisOutputSpec

    def _run_interface(self, runtime):
        from dipy.tracking.utils import move_streamlines, \
            affine_from_fsl_mat_file
        dx, dy, dz = get_data_dims(self.inputs.image_file)
        vx, vy, vz = get_vox_dims(self.inputs.image_file)
        image_file = nb.load(self.inputs.image_file)
        affine = image_file.affine
        out_filename = op.abspath(self.inputs.out_filename)

        # Reads MRTrix tracks
        header, streamlines = read_mrtrix_tracks(
            self.inputs.in_file, as_generator=True)
        iflogger.info('MRTrix Header:')
        iflogger.info(header)
        # Writes to Trackvis
        trk_header = nb.trackvis.empty_header()
        trk_header['dim'] = [dx, dy, dz]
        trk_header['voxel_size'] = [vx, vy, vz]
        trk_header['n_count'] = header['count']

        if isdefined(self.inputs.matrix_file) and isdefined(
                self.inputs.registration_image_file):
            iflogger.info('Applying transformation from matrix file %s',
                          self.inputs.matrix_file)
            xfm = np.genfromtxt(self.inputs.matrix_file)
            iflogger.info(xfm)
            registration_image_file = nb.load(
                self.inputs.registration_image_file)
            reg_affine = registration_image_file.affine
            r_dx, r_dy, r_dz = get_data_dims(
                self.inputs.registration_image_file)
            r_vx, r_vy, r_vz = get_vox_dims(
                self.inputs.registration_image_file)
            iflogger.info('Using affine from registration image file %s',
                          self.inputs.registration_image_file)
            iflogger.info(reg_affine)
            trk_header['vox_to_ras'] = reg_affine
            trk_header['dim'] = [r_dx, r_dy, r_dz]
            trk_header['voxel_size'] = [r_vx, r_vy, r_vz]

            affine = np.dot(affine, np.diag(1. / np.array([vx, vy, vz, 1])))
            transformed_streamlines = transform_to_affine(
                streamlines, trk_header, affine)

            aff = affine_from_fsl_mat_file(xfm, [vx, vy, vz],
                                           [r_vx, r_vy, r_vz])
            iflogger.info(aff)

            axcode = aff2axcodes(reg_affine)
            trk_header['voxel_order'] = axcode[0] + axcode[1] + axcode[2]

            final_streamlines = move_streamlines(transformed_streamlines, aff)
            trk_tracks = ((ii, None, None) for ii in final_streamlines)
            trk.write(out_filename, trk_tracks, trk_header)
            iflogger.info('Saving transformed Trackvis file as %s',
                          out_filename)
            iflogger.info('New TrackVis Header:')
            iflogger.info(trk_header)
        else:
            iflogger.info(
                'Applying transformation from scanner coordinates to %s',
                self.inputs.image_file)
            axcode = aff2axcodes(affine)
            trk_header['voxel_order'] = axcode[0] + axcode[1] + axcode[2]
            trk_header['vox_to_ras'] = affine
            transformed_streamlines = transform_to_affine(
                streamlines, trk_header, affine)
            trk_tracks = ((ii, None, None) for ii in transformed_streamlines)
            trk.write(out_filename, trk_tracks, trk_header)
            iflogger.info('Saving Trackvis file as %s', out_filename)
            iflogger.info('TrackVis Header:')
            iflogger.info(trk_header)
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_file'] = op.abspath(self.inputs.out_filename)
        return outputs

    def _gen_filename(self, name):
        if name == 'out_filename':
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + '.trk'



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class gtractTransformToDisplacementFieldInputSpec(CommandLineInputSpec):
    inputTransform = File(
        desc="Input Transform File Name",
        exists=True,
        argstr="--inputTransform %s")
    inputReferenceVolume = File(
        desc=
        "Required: input image file name to exemplify the anatomical space over which to vcl_express the transform as a displacement field.",
        exists=True,
        argstr="--inputReferenceVolume %s")
    outputDeformationFieldVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output deformation field",
        argstr="--outputDeformationFieldVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractTransformToDisplacementFieldOutputSpec(TraitedSpec):
    outputDeformationFieldVolume = File(
        desc="Output deformation field", exists=True)


class gtractTransformToDisplacementField(SEMLikeCommandLine):
    """title: Create Displacement Field

category: Diffusion.GTRACT

description: This program will compute forward deformation from the given Transform. The size of the DF is equal to MNI space

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta, Madhura Ingalhalikar, and Greg Harris

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractTransformToDisplacementFieldInputSpec
    output_spec = gtractTransformToDisplacementFieldOutputSpec
    _cmd = " gtractTransformToDisplacementField "
    _outputs_filenames = {
        'outputDeformationFieldVolume': 'outputDeformationFieldVolume.nii'
    }
    _redirect_x = False


class gtractInvertBSplineTransformInputSpec(CommandLineInputSpec):
    inputReferenceVolume = File(
        desc=
        "Required: input image file name to exemplify the anatomical space to interpolate over.",
        exists=True,
        argstr="--inputReferenceVolume %s")
    inputTransform = File(
        desc="Required: input B-Spline transform file name",
        exists=True,
        argstr="--inputTransform %s")
    outputTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output transform file name",
        argstr="--outputTransform %s")
    landmarkDensity = InputMultiPath(
        traits.Int,
        desc="Number of landmark subdivisions in all 3 directions",
        sep=",",
        argstr="--landmarkDensity %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractInvertBSplineTransformOutputSpec(TraitedSpec):
    outputTransform = File(
        desc="Required: output transform file name", exists=True)


class gtractInvertBSplineTransform(SEMLikeCommandLine):
    """title: B-Spline Transform Inversion

category: Diffusion.GTRACT

description: This program will invert a B-Spline transform using a thin-plate spline approximation.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractInvertBSplineTransformInputSpec
    output_spec = gtractInvertBSplineTransformOutputSpec
    _cmd = " gtractInvertBSplineTransform "
    _outputs_filenames = {'outputTransform': 'outputTransform.h5'}
    _redirect_x = False


class gtractConcatDwiInputSpec(CommandLineInputSpec):
    inputVolume = InputMultiPath(
        File(exists=True),
        desc=
        "Required: input file containing the first diffusion weighted image",
        argstr="--inputVolume %s...")
    ignoreOrigins = traits.Bool(
        desc=
        "If image origins are different force all images to origin of first image",
        argstr="--ignoreOrigins ")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the combined diffusion weighted images.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractConcatDwiOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the combined diffusion weighted images.",
        exists=True)


class gtractConcatDwi(SEMLikeCommandLine):
    """title: Concat DWI Images

category: Diffusion.GTRACT

description: This program will concatenate two DTI runs together.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractConcatDwiInputSpec
    output_spec = gtractConcatDwiOutputSpec
    _cmd = " gtractConcatDwi "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractAverageBvaluesInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc=
        "Required: input image file name containing multiple baseline gradients to average",
        exists=True,
        argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing directly averaged baseline images",
        argstr="--outputVolume %s")
    directionsTolerance = traits.Float(
        desc="Tolerance for matching identical gradient direction pairs",
        argstr="--directionsTolerance %f")
    averageB0only = traits.Bool(
        desc=
        "Average only baseline gradients. All other gradient directions are not averaged, but retained in the outputVolume",
        argstr="--averageB0only ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractAverageBvaluesOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing directly averaged baseline images",
        exists=True)


class gtractAverageBvalues(SEMLikeCommandLine):
    """title: Average B-Values

category: Diffusion.GTRACT

description: This program will directly average together the baseline gradients (b value equals 0) within a DWI scan. This is usually used after gtractCoregBvalues.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractAverageBvaluesInputSpec
    output_spec = gtractAverageBvaluesOutputSpec
    _cmd = " gtractAverageBvalues "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractCoregBvaluesInputSpec(CommandLineInputSpec):
    movingVolume = File(
        desc=
        "Required: input moving image file name. In order to register gradients within a scan to its first gradient, set the movingVolume and fixedVolume as the same image.",
        exists=True,
        argstr="--movingVolume %s")
    fixedVolume = File(
        desc=
        "Required: input fixed image file name. It is recommended that this image should either contain or be a b0 image.",
        exists=True,
        argstr="--fixedVolume %s")
    fixedVolumeIndex = traits.Int(
        desc=
        "Index in the fixed image for registration. It is recommended that this image should be a b0 image.",
        argstr="--fixedVolumeIndex %d")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing moving images individually resampled and fit to the specified fixed image index.",
        argstr="--outputVolume %s")
    outputTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Registration 3D transforms concatenated in a single output file.  There are no tools that can use this, but can be used for debugging purposes.",
        argstr="--outputTransform %s")
    eddyCurrentCorrection = traits.Bool(
        desc=
        "Flag to perform eddy current corection in addition to motion correction (recommended)",
        argstr="--eddyCurrentCorrection ")
    numberOfIterations = traits.Int(
        desc="Number of iterations in each 3D fit",
        argstr="--numberOfIterations %d")
    numberOfSpatialSamples = traits.Int(
        desc=
        "The number of voxels sampled for mutual information computation.  Increase this for a slower, more careful fit. NOTE that it is suggested to use samplingPercentage instead of this option. However, if set, it overwrites the samplingPercentage option.  ",
        argstr="--numberOfSpatialSamples %d")
    samplingPercentage = traits.Float(
        desc=
        "This is a number in (0.0,1.0] interval that shows the percentage of the input fixed image voxels that are sampled for mutual information computation.  Increase this for a slower, more careful fit. You can also limit the sampling focus with ROI masks and ROIAUTO mask generation. The default is to use approximately 5% of voxels (for backwards compatibility 5% ~= 500000/(256*256*256)). Typical values range from 1% for low detail images to 20% for high detail images.",
        argstr="--samplingPercentage %f")
    relaxationFactor = traits.Float(
        desc=
        "Fraction of gradient from Jacobian to attempt to move in each 3D fit step (adjust when eddyCurrentCorrection is enabled; suggested value = 0.25)",
        argstr="--relaxationFactor %f")
    maximumStepSize = traits.Float(
        desc=
        "Maximum permitted step size to move in each 3D fit step (adjust when eddyCurrentCorrection is enabled; suggested value = 0.1)",
        argstr="--maximumStepSize %f")
    minimumStepSize = traits.Float(
        desc=
        "Minimum required step size to move in each 3D fit step without converging -- decrease this to make the fit more exacting",
        argstr="--minimumStepSize %f")
    spatialScale = traits.Float(
        desc=
        "How much to scale up changes in position compared to unit rotational changes in radians -- decrease this to put more rotation in the fit",
        argstr="--spatialScale %f")
    registerB0Only = traits.Bool(
        desc="Register the B0 images only", argstr="--registerB0Only ")
    debugLevel = traits.Int(
        desc=
        "Display debug messages, and produce debug intermediate results.  0=OFF, 1=Minimal, 10=Maximum debugging.",
        argstr="--debugLevel %d")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractCoregBvaluesOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing moving images individually resampled and fit to the specified fixed image index.",
        exists=True)
    outputTransform = File(
        desc=
        "Registration 3D transforms concatenated in a single output file.  There are no tools that can use this, but can be used for debugging purposes.",
        exists=True)


class gtractCoregBvalues(SEMLikeCommandLine):
    """title: Coregister B-Values

category: Diffusion.GTRACT

description: This step should be performed after converting DWI scans from DICOM to NRRD format. This program will register all gradients in a NRRD diffusion weighted 4D vector image (moving image) to a specified index in a fixed image. It also supports co-registration with a T2 weighted image or field map in the same plane as the DWI data. The fixed image for the registration should be a b0 image. A mutual information metric cost function is used for the registration because of the differences in signal intensity as a result of the diffusion gradients. The full affine allows the registration procedure to correct for eddy current distortions that may exist in the data. If the eddyCurrentCorrection is enabled, relaxationFactor (0.25) and maximumStepSize (0.1) should be adjusted.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractCoregBvaluesInputSpec
    output_spec = gtractCoregBvaluesOutputSpec
    _cmd = " gtractCoregBvalues "
    _outputs_filenames = {
        'outputVolume': 'outputVolume.nrrd',
        'outputTransform': 'outputTransform.h5'
    }
    _redirect_x = False


class gtractResampleAnisotropyInputSpec(CommandLineInputSpec):
    inputAnisotropyVolume = File(
        desc="Required: input file containing the anisotropy image",
        exists=True,
        argstr="--inputAnisotropyVolume %s")
    inputAnatomicalVolume = File(
        desc=
        "Required: input file containing the anatomical image whose characteristics will be cloned.",
        exists=True,
        argstr="--inputAnatomicalVolume %s")
    inputTransform = File(
        desc="Required: input Rigid OR Bspline transform file name",
        exists=True,
        argstr="--inputTransform %s")
    transformType = traits.Enum(
        "Rigid",
        "B-Spline",
        desc="Transform type: Rigid, B-Spline",
        argstr="--transformType %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the resampled transformed anisotropy image.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractResampleAnisotropyOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the resampled transformed anisotropy image.",
        exists=True)


class gtractResampleAnisotropy(SEMLikeCommandLine):
    """title: Resample Anisotropy

category: Diffusion.GTRACT

description: This program will resample a floating point image using either the Rigid or B-Spline transform. You may want to save the aligned B0 image after each of the anisotropy map co-registration steps with the anatomical image to check the registration quality with another tool.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractResampleAnisotropyInputSpec
    output_spec = gtractResampleAnisotropyOutputSpec
    _cmd = " gtractResampleAnisotropy "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractResampleCodeImageInputSpec(CommandLineInputSpec):
    inputCodeVolume = File(
        desc="Required: input file containing the code image",
        exists=True,
        argstr="--inputCodeVolume %s")
    inputReferenceVolume = File(
        desc=
        "Required: input file containing the standard image to clone the characteristics of.",
        exists=True,
        argstr="--inputReferenceVolume %s")
    inputTransform = File(
        desc="Required: input Rigid or Inverse-B-Spline transform file name",
        exists=True,
        argstr="--inputTransform %s")
    transformType = traits.Enum(
        "Rigid",
        "Affine",
        "B-Spline",
        "Inverse-B-Spline",
        "None",
        desc="Transform type: Rigid or Inverse-B-Spline",
        argstr="--transformType %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the resampled code image in acquisition space.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractResampleCodeImageOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the resampled code image in acquisition space.",
        exists=True)


class gtractResampleCodeImage(SEMLikeCommandLine):
    """title: Resample Code Image

category: Diffusion.GTRACT

description: This program will resample a short integer code image using either the Rigid or Inverse-B-Spline transform.  The reference image is the DTI tensor anisotropy image space, and the input code image is in anatomical space.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractResampleCodeImageInputSpec
    output_spec = gtractResampleCodeImageOutputSpec
    _cmd = " gtractResampleCodeImage "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractCopyImageOrientationInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc=
        "Required: input file containing the signed short image to reorient without resampling.",
        exists=True,
        argstr="--inputVolume %s")
    inputReferenceVolume = File(
        desc="Required: input file containing orietation that will be cloned.",
        exists=True,
        argstr="--inputReferenceVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD or Nifti file containing the reoriented image in reference image space.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractCopyImageOrientationOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD or Nifti file containing the reoriented image in reference image space.",
        exists=True)


class gtractCopyImageOrientation(SEMLikeCommandLine):
    """title: Copy Image Orientation

category: Diffusion.GTRACT

description: This program will copy the orientation from the reference image into the moving image. Currently, the registration process requires that the diffusion weighted images and the anatomical images have the same image orientation (i.e. Axial, Coronal, Sagittal). It is suggested that you copy the image orientation from the diffusion weighted images and apply this to the anatomical image. This image can be subsequently removed after the registration step is complete. We anticipate that this limitation will be removed in future versions of the registration programs.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractCopyImageOrientationInputSpec
    output_spec = gtractCopyImageOrientationOutputSpec
    _cmd = " gtractCopyImageOrientation "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractCreateGuideFiberInputSpec(CommandLineInputSpec):
    inputFiber = File(
        desc="Required: input fiber tract file name",
        exists=True,
        argstr="--inputFiber %s")
    numberOfPoints = traits.Int(
        desc="Number of points in output guide fiber",
        argstr="--numberOfPoints %d")
    outputFiber = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output guide fiber file name",
        argstr="--outputFiber %s")
    writeXMLPolyDataFile = traits.Bool(
        desc=
        "Flag to make use of XML files when reading and writing vtkPolyData.",
        argstr="--writeXMLPolyDataFile ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractCreateGuideFiberOutputSpec(TraitedSpec):
    outputFiber = File(
        desc="Required: output guide fiber file name", exists=True)


class gtractCreateGuideFiber(SEMLikeCommandLine):
    """title: Create Guide Fiber

category: Diffusion.GTRACT

description: This program will create a guide fiber by averaging fibers from a previously generated tract.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractCreateGuideFiberInputSpec
    output_spec = gtractCreateGuideFiberOutputSpec
    _cmd = " gtractCreateGuideFiber "
    _outputs_filenames = {'outputFiber': 'outputFiber.vtk'}
    _redirect_x = False


class gtractAnisotropyMapInputSpec(CommandLineInputSpec):
    inputTensorVolume = File(
        desc="Required: input file containing the diffusion tensor image",
        exists=True,
        argstr="--inputTensorVolume %s")
    anisotropyType = traits.Enum(
        "ADC",
        "FA",
        "RA",
        "VR",
        "AD",
        "RD",
        "LI",
        desc="Anisotropy Mapping Type: ADC, FA, RA, VR, AD, RD, LI",
        argstr="--anisotropyType %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the selected kind of anisotropy scalar.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractAnisotropyMapOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the selected kind of anisotropy scalar.",
        exists=True)


class gtractAnisotropyMap(SEMLikeCommandLine):
    """title: Anisotropy Map

category: Diffusion.GTRACT

description: This program will generate a scalar map of anisotropy, given a tensor representation. Anisotropy images are used for fiber tracking, but the anisotropy scalars are not defined along the path. Instead, the tensor representation is included as point data allowing all of these metrics to be computed using only the fiber tract point data. The images can be saved in any ITK supported format, but it is suggested that you use an image format that supports the definition of the image origin. This includes NRRD, NifTI, and Meta formats. These images can also be used for scalar analysis including regional anisotropy measures or VBM style analysis.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractAnisotropyMapInputSpec
    output_spec = gtractAnisotropyMapOutputSpec
    _cmd = " gtractAnisotropyMap "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractClipAnisotropyInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image file name",
        exists=True,
        argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the clipped anisotropy image",
        argstr="--outputVolume %s")
    clipFirstSlice = traits.Bool(
        desc="Clip the first slice of the anisotropy image",
        argstr="--clipFirstSlice ")
    clipLastSlice = traits.Bool(
        desc="Clip the last slice of the anisotropy image",
        argstr="--clipLastSlice ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractClipAnisotropyOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the clipped anisotropy image",
        exists=True)


class gtractClipAnisotropy(SEMLikeCommandLine):
    """title: Clip Anisotropy

category: Diffusion.GTRACT

description: This program will zero the first and/or last slice of an anisotropy image, creating a clipped anisotropy image.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractClipAnisotropyInputSpec
    output_spec = gtractClipAnisotropyOutputSpec
    _cmd = " gtractClipAnisotropy "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractResampleB0InputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input file containing the 4D image",
        exists=True,
        argstr="--inputVolume %s")
    inputAnatomicalVolume = File(
        desc=
        "Required: input file containing the anatomical image defining the origin, spacing and size of the resampled image (template)",
        exists=True,
        argstr="--inputAnatomicalVolume %s")
    inputTransform = File(
        desc="Required: input Rigid OR Bspline transform file name",
        exists=True,
        argstr="--inputTransform %s")
    vectorIndex = traits.Int(
        desc="Index in the diffusion weighted image set for the B0 image",
        argstr="--vectorIndex %d")
    transformType = traits.Enum(
        "Rigid",
        "B-Spline",
        desc="Transform type: Rigid, B-Spline",
        argstr="--transformType %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the resampled input image.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractResampleB0OutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the resampled input image.",
        exists=True)


class gtractResampleB0(SEMLikeCommandLine):
    """title: Resample B0

category: Diffusion.GTRACT

description: This program will resample a signed short image using either a Rigid or B-Spline transform. The user must specify a template image that will be used to define the origin, orientation, spacing, and size of the resampled image.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractResampleB0InputSpec
    output_spec = gtractResampleB0OutputSpec
    _cmd = " gtractResampleB0 "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractInvertRigidTransformInputSpec(CommandLineInputSpec):
    inputTransform = File(
        desc="Required: input rigid transform file name",
        exists=True,
        argstr="--inputTransform %s")
    outputTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output transform file name",
        argstr="--outputTransform %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractInvertRigidTransformOutputSpec(TraitedSpec):
    outputTransform = File(
        desc="Required: output transform file name", exists=True)


class gtractInvertRigidTransform(SEMLikeCommandLine):
    """title: Rigid Transform Inversion

category: Diffusion.GTRACT

description: This program will invert a Rigid transform.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractInvertRigidTransformInputSpec
    output_spec = gtractInvertRigidTransformOutputSpec
    _cmd = " gtractInvertRigidTransform "
    _outputs_filenames = {'outputTransform': 'outputTransform.h5'}
    _redirect_x = False


class gtractImageConformityInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc=
        "Required: input file containing the signed short image to reorient without resampling.",
        exists=True,
        argstr="--inputVolume %s")
    inputReferenceVolume = File(
        desc=
        "Required: input file containing the standard image to clone the characteristics of.",
        exists=True,
        argstr="--inputReferenceVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output Nrrd or Nifti file containing the reoriented image in reference image space.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractImageConformityOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output Nrrd or Nifti file containing the reoriented image in reference image space.",
        exists=True)


class gtractImageConformity(SEMLikeCommandLine):
    """title: Image Conformity

category: Diffusion.GTRACT

description: This program will straighten out the Direction and Origin to match the Reference Image.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractImageConformityInputSpec
    output_spec = gtractImageConformityOutputSpec
    _cmd = " gtractImageConformity "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class compareTractInclusionInputSpec(CommandLineInputSpec):
    testFiber = File(
        desc="Required: test fiber tract file name",
        exists=True,
        argstr="--testFiber %s")
    standardFiber = File(
        desc="Required: standard fiber tract file name",
        exists=True,
        argstr="--standardFiber %s")
    closeness = traits.Float(
        desc=
        "Closeness of every test fiber to some fiber in the standard tract, computed as a sum of squares of spatial differences of standard points",
        argstr="--closeness %f")
    numberOfPoints = traits.Int(
        desc="Number of points in comparison fiber pairs",
        argstr="--numberOfPoints %d")
    testForBijection = traits.Bool(
        desc="Flag to apply the closeness criterion both ways",
        argstr="--testForBijection ")
    testForFiberCardinality = traits.Bool(
        desc="Flag to require the same number of fibers in both tracts",
        argstr="--testForFiberCardinality ")
    writeXMLPolyDataFile = traits.Bool(
        desc=
        "Flag to make use of XML files when reading and writing vtkPolyData.",
        argstr="--writeXMLPolyDataFile ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class compareTractInclusionOutputSpec(TraitedSpec):
    pass


class compareTractInclusion(SEMLikeCommandLine):
    """title: Compare Tracts

category: Diffusion.GTRACT

description: This program will halt with a status code indicating whether a test tract is nearly enough included in a standard tract in the sense that every fiber in the test tract has a low enough sum of squares distance to some fiber in the standard tract modulo spline resampling of every fiber to a fixed number of points.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = compareTractInclusionInputSpec
    output_spec = compareTractInclusionOutputSpec
    _cmd = " compareTractInclusion "
    _outputs_filenames = {}
    _redirect_x = False


class gtractFastMarchingTrackingInputSpec(CommandLineInputSpec):
    inputTensorVolume = File(
        desc="Required: input tensor image file name",
        exists=True,
        argstr="--inputTensorVolume %s")
    inputAnisotropyVolume = File(
        desc="Required: input anisotropy image file name",
        exists=True,
        argstr="--inputAnisotropyVolume %s")
    inputCostVolume = File(
        desc="Required: input vcl_cost image file name",
        exists=True,
        argstr="--inputCostVolume %s")
    inputStartingSeedsLabelMapVolume = File(
        desc="Required: input starting seeds LabelMap image file name",
        exists=True,
        argstr="--inputStartingSeedsLabelMapVolume %s")
    startingSeedsLabel = traits.Int(
        desc="Label value for Starting Seeds",
        argstr="--startingSeedsLabel %d")
    outputTract = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output vtkPolydata file containing tract lines and the point data collected along them.",
        argstr="--outputTract %s")
    writeXMLPolyDataFile = traits.Bool(
        desc="Flag to make use of the XML format for vtkPolyData fiber tracts.",
        argstr="--writeXMLPolyDataFile ")
    numberOfIterations = traits.Int(
        desc="Number of iterations used for the optimization",
        argstr="--numberOfIterations %d")
    seedThreshold = traits.Float(
        desc="Anisotropy threshold used for seed selection",
        argstr="--seedThreshold %f")
    trackingThreshold = traits.Float(
        desc="Anisotropy threshold used for fiber tracking",
        argstr="--trackingThreshold %f")
    costStepSize = traits.Float(
        desc="Cost image sub-voxel sampling", argstr="--costStepSize %f")
    maximumStepSize = traits.Float(
        desc="Maximum step size to move when tracking",
        argstr="--maximumStepSize %f")
    minimumStepSize = traits.Float(
        desc="Minimum step size to move when tracking",
        argstr="--minimumStepSize %f")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractFastMarchingTrackingOutputSpec(TraitedSpec):
    outputTract = File(
        desc=
        "Required: name of output vtkPolydata file containing tract lines and the point data collected along them.",
        exists=True)


class gtractFastMarchingTracking(SEMLikeCommandLine):
    """title: Fast Marching Tracking

category: Diffusion.GTRACT

description: This program will use a fast marching fiber tracking algorithm to identify fiber tracts from a tensor image. This program is the second portion of the algorithm. The user must first run gtractCostFastMarching to generate the vcl_cost image. The second step of the algorithm implemented here is a gradient descent soplution from the defined ending region back to the seed points specified in gtractCostFastMarching. This algorithm is roughly based on the work by G. Parker et al. from IEEE Transactions On Medical Imaging, 21(5): 505-512, 2002. An additional feature of including anisotropy into the vcl_cost function calculation is included.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris. The original code here was developed by Daisy Espino.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractFastMarchingTrackingInputSpec
    output_spec = gtractFastMarchingTrackingOutputSpec
    _cmd = " gtractFastMarchingTracking "
    _outputs_filenames = {'outputTract': 'outputTract.vtk'}
    _redirect_x = False


class gtractInvertDisplacementFieldInputSpec(CommandLineInputSpec):
    baseImage = File(
        desc=
        "Required: base image used to define the size of the inverse field",
        exists=True,
        argstr="--baseImage %s")
    deformationImage = File(
        desc="Required: Displacement field image",
        exists=True,
        argstr="--deformationImage %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: Output deformation field",
        argstr="--outputVolume %s")
    subsamplingFactor = traits.Int(
        desc="Subsampling factor for the deformation field",
        argstr="--subsamplingFactor %d")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractInvertDisplacementFieldOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: Output deformation field", exists=True)


class gtractInvertDisplacementField(SEMLikeCommandLine):
    """title: Invert Displacement Field

category: Diffusion.GTRACT

description: This program will invert a deformatrion field. The size of the deformation field is defined by an example image provided by the user

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractInvertDisplacementFieldInputSpec
    output_spec = gtractInvertDisplacementFieldOutputSpec
    _cmd = " gtractInvertDisplacementField "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False


class gtractCoRegAnatomyInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc=
        "Required: input vector image file name. It is recommended that the input volume is the skull stripped baseline image of the DWI scan.",
        exists=True,
        argstr="--inputVolume %s")
    inputAnatomicalVolume = File(
        desc=
        "Required: input anatomical image file name. It is recommended that that the input anatomical image has been skull stripped and has the same orientation as the DWI scan.",
        exists=True,
        argstr="--inputAnatomicalVolume %s")
    vectorIndex = traits.Int(
        desc=
        "Vector image index in the moving image (within the DWI) to be used for registration.",
        argstr="--vectorIndex %d")
    inputRigidTransform = File(
        desc=
        "Required (for B-Spline type co-registration): input rigid transform file name. Used as a starting point for the anatomical B-Spline registration.",
        exists=True,
        argstr="--inputRigidTransform %s")
    outputTransformName = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: filename for the  fit transform.",
        argstr="--outputTransformName %s")
    transformType = traits.Enum(
        "Rigid",
        "Bspline",
        desc="Transform Type: Rigid|Bspline",
        argstr="--transformType %s")
    numberOfIterations = traits.Int(
        desc="Number of iterations in the selected 3D fit",
        argstr="--numberOfIterations %d")
    gridSize = InputMultiPath(
        traits.Int,
        desc="Number of grid subdivisions in all 3 directions",
        sep=",",
        argstr="--gridSize %s")
    borderSize = traits.Int(desc="Size of border", argstr="--borderSize %d")
    numberOfHistogramBins = traits.Int(
        desc="Number of histogram bins", argstr="--numberOfHistogramBins %d")
    spatialScale = traits.Int(
        desc=
        "Scales the number of voxels in the image by this value to specify the number of voxels used in the registration",
        argstr="--spatialScale %d")
    convergence = traits.Float(
        desc="Convergence Factor", argstr="--convergence %f")
    gradientTolerance = traits.Float(
        desc="Gradient Tolerance", argstr="--gradientTolerance %f")
    maxBSplineDisplacement = traits.Float(
        desc=
        " Sets the maximum allowed displacements in image physical coordinates for BSpline control grid along each axis.  A value of 0.0 indicates that the problem should be unbounded.  NOTE:  This only constrains the BSpline portion, and does not limit the displacement from the associated bulk transform.  This can lead to a substantial reduction in computation time in the BSpline optimizer.,       ",
        argstr="--maxBSplineDisplacement %f")
    maximumStepSize = traits.Float(
        desc="Maximum permitted step size to move in the selected 3D fit",
        argstr="--maximumStepSize %f")
    minimumStepSize = traits.Float(
        desc=
        "Minimum required step size to move in the selected 3D fit without converging -- decrease this to make the fit more exacting",
        argstr="--minimumStepSize %f")
    translationScale = traits.Float(
        desc=
        "How much to scale up changes in position compared to unit rotational changes in radians -- decrease this to put more translation in the fit",
        argstr="--translationScale %f")
    relaxationFactor = traits.Float(
        desc=
        "Fraction of gradient from Jacobian to attempt to move in the selected 3D fit",
        argstr="--relaxationFactor %f")
    numberOfSamples = traits.Int(
        desc=
        "The number of voxels sampled for mutual information computation.  Increase this for a slower, more careful fit. NOTE that it is suggested to use samplingPercentage instead of this option. However, if set, it overwrites the samplingPercentage option.  ",
        argstr="--numberOfSamples %d")
    samplingPercentage = traits.Float(
        desc=
        "This is a number in (0.0,1.0] interval that shows the percentage of the input fixed image voxels that are sampled for mutual information computation.  Increase this for a slower, more careful fit. You can also limit the sampling focus with ROI masks and ROIAUTO mask generation. The default is to use approximately 5% of voxels (for backwards compatibility 5% ~= 500000/(256*256*256)). Typical values range from 1% for low detail images to 20% for high detail images.",
        argstr="--samplingPercentage %f")
    useMomentsAlign = traits.Bool(
        desc=
        "MomentsAlign assumes that the center of mass of the images represent similar structures.  Perform a MomentsAlign registration as part of the sequential registration steps.   This option MUST come first, and CAN NOT be used with either CenterOfHeadLAlign, GeometryAlign, or initialTransform file.  This family of options superceeds the use of transformType if any of them are set.",
        argstr="--useMomentsAlign ")
    useGeometryAlign = traits.Bool(
        desc=
        "GeometryAlign on assumes that the center of the voxel lattice of the images represent similar structures. Perform a GeometryCenterAlign registration as part of the sequential registration steps.   This option MUST come first, and CAN NOT be used with either MomentsAlign, CenterOfHeadAlign, or initialTransform file.  This family of options superceeds the use of transformType if any of them are set.",
        argstr="--useGeometryAlign ")
    useCenterOfHeadAlign = traits.Bool(
        desc=
        "CenterOfHeadAlign attempts to find a hemisphere full of foreground voxels from the superior direction as an estimate of where the center of a head shape would be to drive a center of mass estimate.  Perform a CenterOfHeadAlign registration as part of the sequential registration steps.   This option MUST come first, and CAN NOT be used with either MomentsAlign, GeometryAlign, or initialTransform file.  This family of options superceeds the use of transformType if any of them are set.",
        argstr="--useCenterOfHeadAlign ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractCoRegAnatomyOutputSpec(TraitedSpec):
    outputTransformName = File(
        desc="Required: filename for the  fit transform.", exists=True)


class gtractCoRegAnatomy(SEMLikeCommandLine):
    """title: Coregister B0 to Anatomy B-Spline

category: Diffusion.GTRACT

description: This program will register a Nrrd diffusion weighted 4D vector image to a fixed anatomical image. Two registration methods are supported for alignment with anatomical images: Rigid and B-Spline. The rigid registration performs a rigid body registration with the anatomical images and should be done as well to initialize the B-Spline transform. The B-SPline transform is the deformable transform, where the user can control the amount of deformation based on the number of control points as well as the maximum distance that these points can move. The B-Spline registration places a low dimensional grid in the image, which is deformed. This allows for some susceptibility related distortions to be removed from the diffusion weighted images. In general the amount of motion in the slice selection and read-out directions direction should be kept low. The distortion is in the phase encoding direction in the images. It is recommended that skull stripped (i.e. image containing only brain with skull removed) images shoud be used for image co-registration with the B-Spline transform.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractCoRegAnatomyInputSpec
    output_spec = gtractCoRegAnatomyOutputSpec
    _cmd = " gtractCoRegAnatomy "
    _outputs_filenames = {'outputTransformName': 'outputTransformName.h5'}
    _redirect_x = False


class gtractResampleDWIInPlaceInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image is a 4D NRRD image.",
        exists=True,
        argstr="--inputVolume %s")
    referenceVolume = File(
        desc=
        "If provided, resample to the final space of the referenceVolume 3D data set.",
        exists=True,
        argstr="--referenceVolume %s")
    outputResampledB0 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Convenience function for extracting the first index location (assumed to be the B0)",
        argstr="--outputResampledB0 %s")
    inputTransform = File(
        desc=
        "Required: transform file derived from rigid registration of b0 image to reference structural image.",
        exists=True,
        argstr="--inputTransform %s")
    warpDWITransform = File(
        desc="Optional: transform file to warp gradient volumes.",
        exists=True,
        argstr="--warpDWITransform %s")
    debugLevel = traits.Int(
        desc=
        "Display debug messages, and produce debug intermediate results.  0=OFF, 1=Minimal, 10=Maximum debugging.",
        argstr="--debugLevel %d")
    imageOutputSize = InputMultiPath(
        traits.Int,
        desc=
        "The voxel lattice for the output image, padding is added if necessary. NOTE: if 0,0,0, then the inputVolume size is used.",
        sep=",",
        argstr="--imageOutputSize %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: output image (NRRD file) that has been rigidly transformed into the space of the structural image and padded if image padding was changed from 0,0,0 default.",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractResampleDWIInPlaceOutputSpec(TraitedSpec):
    outputResampledB0 = File(
        desc=
        "Convenience function for extracting the first index location (assumed to be the B0)",
        exists=True)
    outputVolume = File(
        desc=
        "Required: output image (NRRD file) that has been rigidly transformed into the space of the structural image and padded if image padding was changed from 0,0,0 default.",
        exists=True)


class gtractResampleDWIInPlace(SEMLikeCommandLine):
    """title: Resample DWI In Place

category: Diffusion.GTRACT

description: Resamples DWI image to structural image.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta, Greg Harris, Hans Johnson, and Joy Matsui.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractResampleDWIInPlaceInputSpec
    output_spec = gtractResampleDWIInPlaceOutputSpec
    _cmd = " gtractResampleDWIInPlace "
    _outputs_filenames = {
        'outputResampledB0': 'outputResampledB0.nii',
        'outputVolume': 'outputVolume.nii'
    }
    _redirect_x = False


class gtractCostFastMarchingInputSpec(CommandLineInputSpec):
    inputTensorVolume = File(
        desc="Required: input tensor image file name",
        exists=True,
        argstr="--inputTensorVolume %s")
    inputAnisotropyVolume = File(
        desc="Required: input anisotropy image file name",
        exists=True,
        argstr="--inputAnisotropyVolume %s")
    inputStartingSeedsLabelMapVolume = File(
        desc="Required: input starting seeds LabelMap image file name",
        exists=True,
        argstr="--inputStartingSeedsLabelMapVolume %s")
    startingSeedsLabel = traits.Int(
        desc="Label value for Starting Seeds",
        argstr="--startingSeedsLabel %d")
    outputCostVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output vcl_cost image",
        argstr="--outputCostVolume %s")
    outputSpeedVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output speed image",
        argstr="--outputSpeedVolume %s")
    anisotropyWeight = traits.Float(
        desc="Anisotropy weight used for vcl_cost function calculations",
        argstr="--anisotropyWeight %f")
    stoppingValue = traits.Float(
        desc="Terminiating value for vcl_cost function estimation",
        argstr="--stoppingValue %f")
    seedThreshold = traits.Float(
        desc="Anisotropy threshold used for seed selection",
        argstr="--seedThreshold %f")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractCostFastMarchingOutputSpec(TraitedSpec):
    outputCostVolume = File(desc="Output vcl_cost image", exists=True)
    outputSpeedVolume = File(desc="Output speed image", exists=True)


class gtractCostFastMarching(SEMLikeCommandLine):
    """title: Cost Fast Marching

category: Diffusion.GTRACT

description: This program will use a fast marching fiber tracking algorithm to identify fiber tracts from a tensor image. This program is the first portion of the algorithm. The user must first run gtractFastMarchingTracking to generate the actual fiber tracts.  This algorithm is roughly based on the work by G. Parker et al. from IEEE Transactions On Medical Imaging, 21(5): 505-512, 2002. An additional feature of including anisotropy into the vcl_cost function calculation is included.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris. The original code here was developed by Daisy Espino.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractCostFastMarchingInputSpec
    output_spec = gtractCostFastMarchingOutputSpec
    _cmd = " gtractCostFastMarching "
    _outputs_filenames = {
        'outputCostVolume': 'outputCostVolume.nrrd',
        'outputSpeedVolume': 'outputSpeedVolume.nrrd'
    }
    _redirect_x = False


class gtractFiberTrackingInputSpec(CommandLineInputSpec):
    inputTensorVolume = File(
        desc=
        "Required (for Free, Streamline, GraphSearch, and Guided fiber tracking methods): input tensor image file name",
        exists=True,
        argstr="--inputTensorVolume %s")
    inputAnisotropyVolume = File(
        desc=
        "Required (for Free, Streamline, GraphSearch, and Guided fiber tracking methods): input anisotropy image file name",
        exists=True,
        argstr="--inputAnisotropyVolume %s")
    inputStartingSeedsLabelMapVolume = File(
        desc=
        "Required (for Free, Streamline, GraphSearch, and Guided fiber tracking methods): input starting seeds LabelMap image file name",
        exists=True,
        argstr="--inputStartingSeedsLabelMapVolume %s")
    startingSeedsLabel = traits.Int(
        desc=
        "Label value for Starting Seeds (required if Label number used to create seed point in Slicer was not 1)",
        argstr="--startingSeedsLabel %d")
    inputEndingSeedsLabelMapVolume = File(
        desc=
        "Required (for Streamline, GraphSearch, and Guided fiber tracking methods): input ending seeds LabelMap image file name",
        exists=True,
        argstr="--inputEndingSeedsLabelMapVolume %s")
    endingSeedsLabel = traits.Int(
        desc=
        "Label value for Ending Seeds (required if Label number used to create seed point in Slicer was not 1)",
        argstr="--endingSeedsLabel %d")
    inputTract = File(
        desc=
        "Required (for Guided fiber tracking method): guide fiber in vtkPolydata file containing one tract line.",
        exists=True,
        argstr="--inputTract %s")
    outputTract = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required (for Free, Streamline, GraphSearch, and Guided fiber tracking methods): name of output vtkPolydata file containing tract lines and the point data collected along them.",
        argstr="--outputTract %s")
    writeXMLPolyDataFile = traits.Bool(
        desc="Flag to make use of the XML format for vtkPolyData fiber tracts.",
        argstr="--writeXMLPolyDataFile ")
    trackingMethod = traits.Enum(
        "Guided",
        "Free",
        "Streamline",
        "GraphSearch",
        desc="Fiber tracking Filter Type: Guided|Free|Streamline|GraphSearch",
        argstr="--trackingMethod %s")
    guidedCurvatureThreshold = traits.Float(
        desc="Guided Curvature Threshold (Degrees)",
        argstr="--guidedCurvatureThreshold %f")
    maximumGuideDistance = traits.Float(
        desc="Maximum distance for using the guide fiber direction",
        argstr="--maximumGuideDistance %f")
    seedThreshold = traits.Float(
        desc=
        "Anisotropy threshold for seed selection (recommended for Free fiber tracking)",
        argstr="--seedThreshold %f")
    trackingThreshold = traits.Float(
        desc=
        "Anisotropy threshold for fiber tracking (anisotropy values of the next point along the path)",
        argstr="--trackingThreshold %f")
    curvatureThreshold = traits.Float(
        desc=
        "Curvature threshold in degrees (recommended for Free  fiber tracking)",
        argstr="--curvatureThreshold %f")
    branchingThreshold = traits.Float(
        desc=
        "Anisotropy Branching threshold (recommended for GraphSearch fiber tracking method)",
        argstr="--branchingThreshold %f")
    maximumBranchPoints = traits.Int(
        desc=
        "Maximum branch points (recommended for GraphSearch fiber tracking method)",
        argstr="--maximumBranchPoints %d")
    useRandomWalk = traits.Bool(
        desc="Flag to use random walk.", argstr="--useRandomWalk ")
    randomSeed = traits.Int(
        desc="Random number generator seed", argstr="--randomSeed %d")
    branchingAngle = traits.Float(
        desc=
        "Branching angle in degrees (recommended for GraphSearch fiber tracking method)",
        argstr="--branchingAngle %f")
    minimumLength = traits.Float(
        desc="Minimum fiber length. Helpful for filtering invalid tracts.",
        argstr="--minimumLength %f")
    maximumLength = traits.Float(
        desc="Maximum fiber length (voxels)", argstr="--maximumLength %f")
    stepSize = traits.Float(
        desc="Fiber tracking step size", argstr="--stepSize %f")
    useLoopDetection = traits.Bool(
        desc="Flag to make use of loop detection.",
        argstr="--useLoopDetection ")
    useTend = traits.Bool(
        desc="Flag to make use of Tend F and Tend G parameters.",
        argstr="--useTend ")
    tendF = traits.Float(desc="Tend F parameter", argstr="--tendF %f")
    tendG = traits.Float(desc="Tend G parameter", argstr="--tendG %f")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractFiberTrackingOutputSpec(TraitedSpec):
    outputTract = File(
        desc=
        "Required (for Free, Streamline, GraphSearch, and Guided fiber tracking methods): name of output vtkPolydata file containing tract lines and the point data collected along them.",
        exists=True)


class gtractFiberTracking(SEMLikeCommandLine):
    """title: Fiber Tracking

category: Diffusion.GTRACT

description: This program implements four fiber tracking methods (Free, Streamline, GraphSearch, Guided). The output of the fiber tracking is vtkPolyData (i.e. Polylines) that can be loaded into Slicer3 for visualization. The poly data can be saved in either old VTK format files (.vtk) or in the new VTK XML format (.xml). The polylines contain point data that defines ther Tensor at each point along the fiber tract. This can then be used to rendered as glyphs in Slicer3 and can be used to define severeal scalar measures without referencing back to the anisotropy images. (1) Free tracking is a basic streamlines algorithm. This is a direct implementation of the method original proposed by Basser et al. The tracking follows the primarty eigenvector. The tracking begins with seed points in the starting region. Only those voxels above the specified anisotropy threshold in the starting region are used as seed points. Tracking terminates either as a result of maximum fiber length, low ansiotropy, or large curvature. This is a great way to explore your data. (2) The streamlines algorithm is a direct implementation of the method originally proposed by Basser et al. The tracking follows the primary eigenvector. The tracking begins with seed points in the starting region. Only those voxels above the specified anisotropy threshold in the starting region are used as seed points. Tracking terminates either by reaching the ending region or reaching some stopping criteria. Stopping criteria are specified using the following parameters: tracking threshold, curvature threshold, and max length. Only paths terminating in the ending region are kept in this method. The TEND algorithm proposed by Lazar et al. (Human Brain Mapping 18:306-321, 2003) has been instrumented. This can be enabled using the --useTend option while performing Streamlines tracking. This utilizes the entire diffusion tensor to deflect the incoming vector instead of simply following the primary eigenvector. The TEND parameters are set using the --tendF and --tendG options. (3) Graph Search tracking is the first step in the full GTRACT algorithm developed by Cheng et al. (NeuroImage 31(3): 1075-1085, 2006) for finding the tracks in a tensor image. This method was developed to generate fibers in a Tensor representation where crossing fibers occur. The graph search algorithm follows the primary eigenvector in non-ambigous regions and utilizes branching and a graph search algorithm in ambigous regions. Ambiguous tracking regions are defined based on two criteria: Branching Al Threshold (anisotropy values below this value and above the traching threshold) and Curvature Major Eigen (angles of the primary eigenvector direction and the current tracking direction). In regions that meet this criteria, two or three tracking paths are considered. The first is the standard primary eigenvector direction. The second is the seconadary eigenvector direction. This is based on the assumption that these regions may be prolate regions. If the Random Walk option is selected then a third direction is also considered. This direction is defined by a cone pointing from the current position to the centroid of the ending region. The interior angle of the cone is specified by the user with the Branch/Guide Angle parameter. A vector contained inside of the cone is selected at random and used as the third direction. This method can also utilize the TEND option where the primary tracking direction is that specified by the TEND method instead of the primary eigenvector. The parameter '--maximumBranchPoints' allows the tracking to have this number of branches being considered at a time. If this number of branch points is exceeded at any time, then the algorithm will revert back to a streamline alogrithm until the number of branches is reduced. This allows the user to constrain the computational complexity of the algorithm. (4) The second phase of the GTRACT algorithm is Guided Tracking. This method incorporates anatomical information about the track orientation using an initial guess of the fiber track. In the originally proposed GTRACT method, this would be created from the fibers resulting from the Graph Search tracking. However, in practice this can be created using any method and could be defined manually. To create the guide fiber the program gtractCreateGuideFiber can be used. This program will load a fiber tract that has been generated and create a centerline representation of the fiber tract (i.e. a single fiber). In this method, the fiber tracking follows the primary eigenvector direction unless it deviates from the guide fiber track by a angle greater than that specified by the '--guidedCurvatureThreshold' parameter. The user must specify the guide fiber when running this program.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta, Greg Harris and Yongqiang Zhao.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractFiberTrackingInputSpec
    output_spec = gtractFiberTrackingOutputSpec
    _cmd = " gtractFiberTracking "
    _outputs_filenames = {'outputTract': 'outputTract.vtk'}
    _redirect_x = False


class extractNrrdVectorIndexInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc=
        "Required: input file containing the vector that will be extracted",
        exists=True,
        argstr="--inputVolume %s")
    vectorIndex = traits.Int(
        desc="Index in the vector image to extract", argstr="--vectorIndex %d")
    setImageOrientation = traits.Enum(
        "AsAcquired",
        "Axial",
        "Coronal",
        "Sagittal",
        desc=
        "Sets the image orientation of the extracted vector (Axial, Coronal, Sagittal)",
        argstr="--setImageOrientation %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the vector image at the given index",
        argstr="--outputVolume %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class extractNrrdVectorIndexOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the vector image at the given index",
        exists=True)


class extractNrrdVectorIndex(SEMLikeCommandLine):
    """title: Extract Nrrd Index

category: Diffusion.GTRACT

description: This program will extract a 3D image (single vector) from a vector 3D image at a given vector index.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = extractNrrdVectorIndexInputSpec
    output_spec = extractNrrdVectorIndexOutputSpec
    _cmd = " extractNrrdVectorIndex "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}
    _redirect_x = False


class gtractResampleFibersInputSpec(CommandLineInputSpec):
    inputForwardDeformationFieldVolume = File(
        desc="Required: input forward deformation field image file name",
        exists=True,
        argstr="--inputForwardDeformationFieldVolume %s")
    inputReverseDeformationFieldVolume = File(
        desc="Required: input reverse deformation field image file name",
        exists=True,
        argstr="--inputReverseDeformationFieldVolume %s")
    inputTract = File(
        desc="Required: name of input vtkPolydata file containing tract lines.",
        exists=True,
        argstr="--inputTract %s")
    outputTract = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output vtkPolydata file containing tract lines and the point data collected along them.",
        argstr="--outputTract %s")
    writeXMLPolyDataFile = traits.Bool(
        desc="Flag to make use of the XML format for vtkPolyData fiber tracts.",
        argstr="--writeXMLPolyDataFile ")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractResampleFibersOutputSpec(TraitedSpec):
    outputTract = File(
        desc=
        "Required: name of output vtkPolydata file containing tract lines and the point data collected along them.",
        exists=True)


class gtractResampleFibers(SEMLikeCommandLine):
    """title: Resample Fibers

category: Diffusion.GTRACT

description: This program will resample a fiber tract with respect to a pair of deformation fields that represent the forward and reverse deformation fields.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractResampleFibersInputSpec
    output_spec = gtractResampleFibersOutputSpec
    _cmd = " gtractResampleFibers "
    _outputs_filenames = {'outputTract': 'outputTract.vtk'}
    _redirect_x = False


class gtractTensorInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc=
        "Required: input image 4D NRRD image. Must contain data based on at least 6 distinct diffusion directions. The inputVolume is allowed to have multiple b0 and gradient direction images. Averaging of the b0 image is done internally in this step. Prior averaging of the DWIs is not required.",
        exists=True,
        argstr="--inputVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc=
        "Required: name of output NRRD file containing the Tensor vector image",
        argstr="--outputVolume %s")
    medianFilterSize = InputMultiPath(
        traits.Int,
        desc="Median filter radius in all 3 directions",
        sep=",",
        argstr="--medianFilterSize %s")
    maskProcessingMode = traits.Enum(
        "NOMASK",
        "ROIAUTO",
        "ROI",
        desc=
        "ROIAUTO:  mask is implicitly defined using a otsu forground and hole filling algorithm. ROI: Uses the masks to define what parts of the image should be used for computing the transform. NOMASK: no mask used",
        argstr="--maskProcessingMode %s")
    maskVolume = File(
        desc="Mask Image, if maskProcessingMode is ROI",
        exists=True,
        argstr="--maskVolume %s")
    backgroundSuppressingThreshold = traits.Int(
        desc=
        "Image threshold to suppress background. This sets a threshold used on the b0 image to remove background voxels from processing. Typically, values of 100 and 500 work well for Siemens and GE DTI data, respectively. Check your data particularly in the globus pallidus to make sure the brain tissue is not being eliminated with this threshold.",
        argstr="--backgroundSuppressingThreshold %d")
    resampleIsotropic = traits.Bool(
        desc=
        "Flag to resample to isotropic voxels. Enabling this feature is recommended if fiber tracking will be performed.",
        argstr="--resampleIsotropic ")
    size = traits.Float(
        desc="Isotropic voxel size to resample to", argstr="--size %f")
    b0Index = traits.Int(
        desc="Index in input vector index to extract", argstr="--b0Index %d")
    applyMeasurementFrame = traits.Bool(
        desc="Flag to apply the measurement frame to the gradient directions",
        argstr="--applyMeasurementFrame ")
    ignoreIndex = InputMultiPath(
        traits.Int,
        desc=
        "Ignore diffusion gradient index. Used to remove specific gradient directions with artifacts.",
        sep=",",
        argstr="--ignoreIndex %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class gtractTensorOutputSpec(TraitedSpec):
    outputVolume = File(
        desc=
        "Required: name of output NRRD file containing the Tensor vector image",
        exists=True)


class gtractTensor(SEMLikeCommandLine):
    """title: Tensor Estimation

category: Diffusion.GTRACT

description: This step will convert a b-value averaged diffusion tensor image to a 3x3 tensor voxel image. This step takes the diffusion tensor image data and generates a tensor representation of the data based on the signal intensity decay, b values applied, and the diffusion difrections. The apparent diffusion coefficient for a given orientation is computed on a pixel-by-pixel basis by fitting the image data (voxel intensities) to the Stejskal-Tanner equation. If at least 6 diffusion directions are used, then the diffusion tensor can be computed. This program uses itk::DiffusionTensor3DReconstructionImageFilter. The user can adjust background threshold, median filter, and isotropic resampling.

version: 4.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:GTRACT

license: http://mri.radiology.uiowa.edu/copyright/GTRACT-Copyright.txt

contributor: This tool was developed by Vincent Magnotta and Greg Harris.

acknowledgements: Funding for this version of the GTRACT program was provided by NIH/NINDS R01NS050568-01A2S1

"""

    input_spec = gtractTensorInputSpec
    output_spec = gtractTensorOutputSpec
    _cmd = " gtractTensor "
    _outputs_filenames = {'outputVolume': 'outputVolume.nrrd'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import BrainExtraction


def test_BrainExtraction_inputs():
    input_map = dict(
        anatomical_image=dict(
            argstr='-a %s',
            extensions=None,
            mandatory=True,
        ),
        args=dict(argstr='%s', ),
        brain_probability_mask=dict(
            argstr='-m %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        brain_template=dict(
            argstr='-e %s',
            extensions=None,
            mandatory=True,
        ),
        debug=dict(argstr='-z 1', ),
        dimension=dict(
            argstr='-d %d',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        extraction_registration_mask=dict(
            argstr='-f %s',
            extensions=None,
        ),
        image_suffix=dict(
            argstr='-s %s',
            usedefault=True,
        ),
        keep_temporary_files=dict(argstr='-k %d', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_prefix=dict(
            argstr='-o %s',
            usedefault=True,
        ),
        use_floatingpoint_precision=dict(argstr='-q %d', ),
        use_random_seeding=dict(argstr='-u %d', ),
    )
    inputs = BrainExtraction.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BrainExtraction_outputs():
    output_map = dict(
        BrainExtractionBrain=dict(extensions=None, ),
        BrainExtractionCSF=dict(extensions=None, ),
        BrainExtractionGM=dict(extensions=None, ),
        BrainExtractionInitialAffine=dict(extensions=None, ),
        BrainExtractionInitialAffineFixed=dict(extensions=None, ),
        BrainExtractionInitialAffineMoving=dict(extensions=None, ),
        BrainExtractionLaplacian=dict(extensions=None, ),
        BrainExtractionMask=dict(extensions=None, ),
        BrainExtractionPrior0GenericAffine=dict(extensions=None, ),
        BrainExtractionPrior1InverseWarp=dict(extensions=None, ),
        BrainExtractionPrior1Warp=dict(extensions=None, ),
        BrainExtractionPriorWarped=dict(extensions=None, ),
        BrainExtractionSegmentation=dict(extensions=None, ),
        BrainExtractionTemplateLaplacian=dict(extensions=None, ),
        BrainExtractionTmp=dict(extensions=None, ),
        BrainExtractionWM=dict(extensions=None, ),
        N4Corrected0=dict(extensions=None, ),
        N4Truncated0=dict(extensions=None, ),
    )
    outputs = BrainExtraction.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..simulate import SimulateMultiTensor


def test_SimulateMultiTensor_inputs():
    input_map = dict(
        baseline=dict(
            extensions=None,
            mandatory=True,
        ),
        bvalues=dict(usedefault=True, ),
        diff_iso=dict(usedefault=True, ),
        diff_sf=dict(usedefault=True, ),
        gradients=dict(extensions=None, ),
        in_bval=dict(extensions=None, ),
        in_bvec=dict(extensions=None, ),
        in_dirs=dict(mandatory=True, ),
        in_frac=dict(mandatory=True, ),
        in_mask=dict(extensions=None, ),
        in_vfms=dict(mandatory=True, ),
        n_proc=dict(usedefault=True, ),
        num_dirs=dict(usedefault=True, ),
        out_bval=dict(
            extensions=None,
            usedefault=True,
        ),
        out_bvec=dict(
            extensions=None,
            usedefault=True,
        ),
        out_file=dict(
            extensions=None,
            usedefault=True,
        ),
        out_mask=dict(
            extensions=None,
            usedefault=True,
        ),
        snr=dict(usedefault=True, ),
    )
    inputs = SimulateMultiTensor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SimulateMultiTensor_outputs():
    output_map = dict(
        out_bval=dict(extensions=None, ),
        out_bvec=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        out_mask=dict(extensions=None, ),
    )
    outputs = SimulateMultiTensor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Nipype base interfaces
----------------------

This module defines the API of all nipype interfaces.

"""
from traits.trait_handlers import TraitDictObject, TraitListObject
from traits.trait_errors import TraitError

from .core import (Interface, BaseInterface, SimpleInterface, CommandLine,
                   StdOutCommandLine, MpiCommandLine, SEMLikeCommandLine,
                   LibraryBaseInterface, PackageInfo)

from .specs import (BaseTraitedSpec, TraitedSpec, DynamicTraitedSpec,
                    BaseInterfaceInputSpec, CommandLineInputSpec,
                    StdOutCommandLineInputSpec)

from .traits_extension import (
    traits, Undefined, isdefined, has_metadata,
    File, ImageFile, Directory,
    Str, DictStrStr,
    OutputMultiObject, InputMultiObject,
    OutputMultiPath, InputMultiPath)

from .support import (Bunch, InterfaceResult, NipypeInterfaceError)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import MedicAlgorithmN3


def test_MedicAlgorithmN3_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inAutomatic=dict(argstr='--inAutomatic %s', ),
        inEnd=dict(argstr='--inEnd %f', ),
        inField=dict(argstr='--inField %f', ),
        inInput=dict(
            argstr='--inInput %s',
            extensions=None,
        ),
        inKernel=dict(argstr='--inKernel %f', ),
        inMaximum=dict(argstr='--inMaximum %d', ),
        inSignal=dict(argstr='--inSignal %f', ),
        inSubsample=dict(argstr='--inSubsample %f', ),
        inWeiner=dict(argstr='--inWeiner %f', ),
        null=dict(argstr='--null %s', ),
        outInhomogeneity=dict(
            argstr='--outInhomogeneity %s',
            hash_files=False,
        ),
        outInhomogeneity2=dict(
            argstr='--outInhomogeneity2 %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = MedicAlgorithmN3.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MedicAlgorithmN3_outputs():
    output_map = dict(
        outInhomogeneity=dict(extensions=None, ),
        outInhomogeneity2=dict(extensions=None, ),
    )
    outputs = MedicAlgorithmN3.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Blob


def test_Blob_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        determinant=dict(argstr='-determinant', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        magnitude=dict(argstr='-magnitude', ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=['input_file'],
            name_template='%s_blob.mnc',
            position=-1,
        ),
        trace=dict(argstr='-trace', ),
        translation=dict(argstr='-translation', ),
    )
    inputs = Blob.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Blob_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Blob.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Testing module for functions and classes from multiproc.py
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import range, open

# Import packages
import os
import sys
from tempfile import mkdtemp
from shutil import rmtree
import pytest

import nipype.pipeline.engine as pe
from nipype.interfaces.utility import Function


def mytestFunction(insum=0):
    '''
    Run a multiprocessing job and spawn child processes.
    '''

    # need to import here since this is executed as an external process
    import multiprocessing
    import os
    import tempfile
    import time

    numberOfThreads = 2

    # list of processes
    t = [None] * numberOfThreads

    # list of alive flags
    a = [None] * numberOfThreads

    # list of tempFiles
    f = [None] * numberOfThreads

    def dummyFunction(filename):
        '''
        This function writes the value 45 to the given filename.
        '''
        j = 0
        for i in range(0, 10):
            j += i

        # j is now 45 (0+1+2+3+4+5+6+7+8+9)

        with open(filename, 'w') as f:
            f.write(str(j))

    for n in range(numberOfThreads):

        # mark thread as alive
        a[n] = True

        # create a temp file to use as the data exchange container
        tmpFile = tempfile.mkstemp('.txt', 'test_engine_')[1]
        f[n] = tmpFile  # keep track of the temp file
        t[n] = multiprocessing.Process(target=dummyFunction, args=(tmpFile, ))
        # fire up the job
        t[n].start()

    # block until all processes are done
    allDone = False
    while not allDone:

        time.sleep(1)

        for n in range(numberOfThreads):

            a[n] = t[n].is_alive()

        if not any(a):
            # if no thread is alive
            allDone = True

    # here, all processes are done

    # read in all temp files and sum them up
    total = insum
    for ff in f:
        with open(ff) as fd:
            total += int(fd.read())
        os.remove(ff)

    return total


def run_multiproc_nondaemon_with_flag(nondaemon_flag):
    '''
    Start a pipe with two nodes using the resource multiproc plugin and
    passing the nondaemon_flag.
    '''

    cur_dir = os.getcwd()
    temp_dir = mkdtemp(prefix='test_engine_')
    os.chdir(temp_dir)

    pipe = pe.Workflow(name='pipe')

    f1 = pe.Node(
        interface=Function(
            function=mytestFunction,
            input_names=['insum'],
            output_names=['sum_out']),
        name='f1')
    f2 = pe.Node(
        interface=Function(
            function=mytestFunction,
            input_names=['insum'],
            output_names=['sum_out']),
        name='f2')

    pipe.connect([(f1, f2, [('sum_out', 'insum')])])
    pipe.base_dir = os.getcwd()
    f1.inputs.insum = 0

    pipe.config['execution']['stop_on_first_crash'] = True

    # execute the pipe using the LegacyMultiProc plugin with 2 processes and the
    # non_daemon flag to enable child processes which start other
    # multiprocessing jobs
    execgraph = pipe.run(
        plugin="LegacyMultiProc",
        plugin_args={
            'n_procs': 2,
            'non_daemon': nondaemon_flag
        })

    names = [
        '.'.join((node._hierarchy, node.name)) for node in execgraph.nodes()
    ]
    node = list(execgraph.nodes())[names.index('pipe.f2')]
    result = node.get_output('sum_out')
    os.chdir(cur_dir)
    rmtree(temp_dir)
    return result


def test_run_multiproc_nondaemon_false():
    '''
    This is the entry point for the test. Two times a pipe of several
    multiprocessing jobs gets executed. First, without the nondaemon flag.
    Second, with the nondaemon flag.

    Since the processes of the pipe start child processes, the execution only
    succeeds when the non_daemon flag is on.
    '''
    shouldHaveFailed = False
    try:
        # with nondaemon_flag = False, the execution should fail
        run_multiproc_nondaemon_with_flag(False)
    except:
        shouldHaveFailed = True
    assert shouldHaveFailed


def test_run_multiproc_nondaemon_true():
    # with nondaemon_flag = True, the execution should succeed
    result = run_multiproc_nondaemon_with_flag(True)
    assert result == 180  # n_procs (2) * numberOfThreads (2) * 45 == 180



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import QualityIndex


def test_QualityIndex_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        autoclip=dict(
            argstr='-autoclip',
            usedefault=True,
            xor=['mask'],
        ),
        automask=dict(
            argstr='-automask',
            usedefault=True,
            xor=['mask'],
        ),
        clip=dict(argstr='-clip %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        interval=dict(
            argstr='-range',
            usedefault=True,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
            xor=['autoclip', 'automask'],
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            keep_extension=False,
            name_source=['in_file'],
            name_template='%s_tqual',
            position=-1,
        ),
        quadrant=dict(
            argstr='-quadrant',
            usedefault=True,
        ),
        spearman=dict(
            argstr='-spearman',
            usedefault=True,
        ),
    )
    inputs = QualityIndex.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_QualityIndex_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = QualityIndex.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Copy


def test_Copy_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=['input_file'],
            name_template='%s_copy.mnc',
            position=-1,
        ),
        pixel_values=dict(
            argstr='-pixel_values',
            xor=('pixel_values', 'real_values'),
        ),
        real_values=dict(
            argstr='-real_values',
            xor=('pixel_values', 'real_values'),
        ),
    )
    inputs = Copy.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Copy_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Copy.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .commandlineonly import fiberstats
from .fiberprocess import fiberprocess
from .fibertrack import fibertrack
from .ukftractography import UKFTractography



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..convert import NIfTIDT2Camino


def test_NIfTIDT2Camino_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bgmask=dict(
            argstr='-bgmask %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-inputfile %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        lns0_file=dict(
            argstr='-lns0 %s',
            extensions=None,
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        s0_file=dict(
            argstr='-s0 %s',
            extensions=None,
        ),
        scaleinter=dict(argstr='-scaleinter %s', ),
        scaleslope=dict(argstr='-scaleslope %s', ),
        uppertriangular=dict(argstr='-uppertriangular %s', ),
    )
    inputs = NIfTIDT2Camino.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_NIfTIDT2Camino_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = NIfTIDT2Camino.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import RobustFOV


def test_RobustFOV_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        brainsize=dict(argstr='-b %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        out_roi=dict(
            argstr='-r %s',
            extensions=None,
            hash_files=False,
            name_source=['in_file'],
            name_template='%s_ROI',
        ),
        out_transform=dict(
            argstr='-m %s',
            extensions=None,
            hash_files=False,
            name_source=['in_file'],
            name_template='%s_to_ROI',
        ),
        output_type=dict(),
    )
    inputs = RobustFOV.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RobustFOV_outputs():
    output_map = dict(
        out_roi=dict(extensions=None, ),
        out_transform=dict(extensions=None, ),
    )
    outputs = RobustFOV.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import CurvatureStats


def test_CurvatureStats_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        copy_inputs=dict(),
        curvfile1=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        curvfile2=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemisphere=dict(
            argstr='%s',
            mandatory=True,
            position=-3,
        ),
        min_max=dict(argstr='-m', ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            hash_files=False,
            name_source=['hemisphere'],
            name_template='%s.curv.stats',
        ),
        subject_id=dict(
            argstr='%s',
            mandatory=True,
            position=-4,
            usedefault=True,
        ),
        subjects_dir=dict(),
        surface=dict(
            argstr='-F %s',
            extensions=None,
        ),
        values=dict(argstr='-G', ),
        write=dict(argstr='--writeCurvatureFiles', ),
    )
    inputs = CurvatureStats.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CurvatureStats_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = CurvatureStats.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import DTLUTGen


def test_DTLUTGen_inputs():
    input_map = dict(
        acg=dict(argstr='-acg', ),
        args=dict(argstr='%s', ),
        bingham=dict(argstr='-bingham', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        frange=dict(
            argstr='-frange %s',
            position=1,
            units='NA',
        ),
        inversion=dict(
            argstr='-inversion %d',
            units='NA',
        ),
        lrange=dict(
            argstr='-lrange %s',
            position=1,
            units='NA',
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        samples=dict(
            argstr='-samples %d',
            units='NA',
        ),
        scheme_file=dict(
            argstr='-schemefile %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        snr=dict(
            argstr='-snr %f',
            units='NA',
        ),
        step=dict(
            argstr='-step %f',
            units='NA',
        ),
        trace=dict(
            argstr='-trace %G',
            units='NA',
        ),
        watson=dict(argstr='-watson', ),
    )
    inputs = DTLUTGen.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DTLUTGen_outputs():
    output_map = dict(dtLUT=dict(extensions=None, ), )
    outputs = DTLUTGen.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import DWIConvert


def test_DWIConvert_inputs():
    input_map = dict(
        allowLossyConversion=dict(argstr='--allowLossyConversion ', ),
        args=dict(argstr='%s', ),
        conversionMode=dict(argstr='--conversionMode %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fMRI=dict(argstr='--fMRI ', ),
        fslNIFTIFile=dict(
            argstr='--fslNIFTIFile %s',
            extensions=None,
        ),
        gradientVectorFile=dict(
            argstr='--gradientVectorFile %s',
            hash_files=False,
        ),
        inputBValues=dict(
            argstr='--inputBValues %s',
            extensions=None,
        ),
        inputBVectors=dict(
            argstr='--inputBVectors %s',
            extensions=None,
        ),
        inputDicomDirectory=dict(argstr='--inputDicomDirectory %s', ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        outputBValues=dict(
            argstr='--outputBValues %s',
            hash_files=False,
        ),
        outputBVectors=dict(
            argstr='--outputBVectors %s',
            hash_files=False,
        ),
        outputDirectory=dict(
            argstr='--outputDirectory %s',
            hash_files=False,
        ),
        outputVolume=dict(
            argstr='--outputVolume %s',
            hash_files=False,
        ),
        smallGradientThreshold=dict(argstr='--smallGradientThreshold %f', ),
        transposeInputBVectors=dict(argstr='--transposeInputBVectors ', ),
        useBMatrixGradientDirections=dict(
            argstr='--useBMatrixGradientDirections ', ),
        useIdentityMeaseurementFrame=dict(
            argstr='--useIdentityMeaseurementFrame ', ),
        writeProtocolGradientsFile=dict(
            argstr='--writeProtocolGradientsFile ', ),
    )
    inputs = DWIConvert.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DWIConvert_outputs():
    output_map = dict(
        gradientVectorFile=dict(extensions=None, ),
        outputBValues=dict(extensions=None, ),
        outputBVectors=dict(extensions=None, ),
        outputDirectory=dict(),
        outputVolume=dict(extensions=None, ),
    )
    outputs = DWIConvert.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from ..base import CommandLineInputSpec, CommandLine, TraitedSpec, File


class Vnifti2ImageInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='-in %s',
        mandatory=True,
        position=1,
        desc='in file')
    attributes = File(
        exists=True, argstr='-attr %s', position=2, desc='attribute file')
    out_file = File(
        name_template="%s.v",
        keep_extension=False,
        argstr='-out %s',
        hash_files=False,
        position=-1,
        desc='output data file',
        name_source=["in_file"])


class Vnifti2ImageOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='Output vista file')


class Vnifti2Image(CommandLine):
    """
    Convert a nifti file into a vista file.

    Example
    -------

    >>> vimage = Vnifti2Image()
    >>> vimage.inputs.in_file = 'image.nii'
    >>> vimage.cmdline
    'vnifti2image -in image.nii -out image.v'
    >>> vimage.run()                                       # doctest: +SKIP
    """

    _cmd = 'vnifti2image'
    input_spec = Vnifti2ImageInputSpec
    output_spec = Vnifti2ImageOutputSpec


class VtoMatInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='-in %s',
        mandatory=True,
        position=1,
        desc='in file')
    out_file = File(
        name_template="%s.mat",
        keep_extension=False,
        argstr='-out %s',
        hash_files=False,
        position=-1,
        desc='output mat file',
        name_source=["in_file"])


class VtoMatOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='Output mat file')


class VtoMat(CommandLine):
    """
    Convert a nifti file into a vista file.

    Example
    -------

    >>> vimage = VtoMat()
    >>> vimage.inputs.in_file = 'image.v'
    >>> vimage.cmdline
    'vtomat -in image.v -out image.mat'
    >>> vimage.run()                                       # doctest: +SKIP
    """

    _cmd = 'vtomat'
    input_spec = VtoMatInputSpec
    output_spec = VtoMatOutputSpec



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import FWHMx


def test_FWHMx_inputs():
    input_map = dict(
        acf=dict(
            argstr='-acf',
            usedefault=True,
        ),
        args=dict(argstr='%s', ),
        arith=dict(
            argstr='-arith',
            xor=['geom'],
        ),
        automask=dict(
            argstr='-automask',
            usedefault=True,
        ),
        combine=dict(argstr='-combine', ),
        compat=dict(argstr='-compat', ),
        demed=dict(
            argstr='-demed',
            xor=['detrend'],
        ),
        detrend=dict(
            argstr='-detrend',
            usedefault=True,
            xor=['demed'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        geom=dict(
            argstr='-geom',
            xor=['arith'],
        ),
        in_file=dict(
            argstr='-input %s',
            extensions=None,
            mandatory=True,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        out_detrend=dict(
            argstr='-detprefix %s',
            extensions=None,
            keep_extension=False,
            name_source='in_file',
            name_template='%s_detrend',
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            keep_extension=False,
            name_source='in_file',
            name_template='%s_fwhmx.out',
            position=-1,
        ),
        out_subbricks=dict(
            argstr='-out %s',
            extensions=None,
            keep_extension=False,
            name_source='in_file',
            name_template='%s_subbricks.out',
        ),
        unif=dict(argstr='-unif', ),
    )
    inputs = FWHMx.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FWHMx_outputs():
    output_map = dict(
        acf_param=dict(),
        fwhm=dict(),
        out_acf=dict(extensions=None, ),
        out_detrend=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        out_subbricks=dict(extensions=None, ),
    )
    outputs = FWHMx.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dcm2nii import Dcm2niix


def test_Dcm2niix_inputs():
    input_map = dict(
        anon_bids=dict(
            argstr='-ba',
            requires=['bids_format'],
        ),
        args=dict(argstr='%s', ),
        bids_format=dict(
            argstr='-b',
            usedefault=True,
        ),
        comment=dict(argstr='-c %s', ),
        compress=dict(
            argstr='-z %s',
            usedefault=True,
        ),
        compression=dict(argstr='-%d', ),
        crop=dict(
            argstr='-x',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        has_private=dict(
            argstr='-t',
            usedefault=True,
        ),
        ignore_deriv=dict(argstr='-i', ),
        merge_imgs=dict(
            argstr='-m',
            usedefault=True,
        ),
        out_filename=dict(argstr='-f %s', ),
        output_dir=dict(
            argstr='-o %s',
            usedefault=True,
        ),
        philips_float=dict(argstr='-p', ),
        series_numbers=dict(argstr='-n %s...', ),
        single_file=dict(
            argstr='-s',
            usedefault=True,
        ),
        source_dir=dict(
            argstr='%s',
            mandatory=True,
            position=-1,
            xor=['source_names'],
        ),
        source_names=dict(
            argstr='%s',
            copyfile=False,
            mandatory=True,
            position=-1,
            xor=['source_dir'],
        ),
        to_nrrd=dict(argstr='-e', ),
        verbose=dict(
            argstr='-v',
            usedefault=True,
        ),
    )
    inputs = Dcm2niix.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Dcm2niix_outputs():
    output_map = dict(
        bids=dict(),
        bvals=dict(),
        bvecs=dict(),
        converted_files=dict(),
    )
    outputs = Dcm2niix.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..label_fusion import LabelFusion


def test_LabelFusion_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        classifier_type=dict(
            argstr='-%s',
            mandatory=True,
            position=2,
        ),
        conv=dict(argstr='-conv %f', ),
        dilation_roi=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        file_to_seg=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            argstr='-in %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        kernel_size=dict(),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        max_iter=dict(argstr='-max_iter %d', ),
        mrf_value=dict(argstr='-MRF_beta %f', ),
        out_file=dict(
            argstr='-out %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s',
        ),
        prob_flag=dict(argstr='-outProb', ),
        prob_update_flag=dict(argstr='-prop_update', ),
        proportion=dict(argstr='-prop %s', ),
        set_pq=dict(argstr='-setPQ %f %f', ),
        sm_ranking=dict(
            argstr='-%s',
            position=3,
            usedefault=True,
        ),
        template_file=dict(extensions=None, ),
        template_num=dict(),
        unc=dict(argstr='-unc', ),
        unc_thresh=dict(argstr='-uncthres %f', ),
        verbose=dict(argstr='-v %s', ),
    )
    inputs = LabelFusion.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_LabelFusion_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = LabelFusion.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import unicode_literals
from builtins import str, bytes
from future import standard_library
standard_library.install_aliases()

import os

from nipype.utils.provenance import ProvStore, safe_encode


def test_provenance(tmpdir):
    from nipype.interfaces.base import CommandLine
    tmpdir.chdir()
    ps = ProvStore()
    results = CommandLine('echo hello').run()
    ps.add_results(results)
    provn = ps.g.get_provn()
    assert 'echo hello' in provn


def test_provenance_exists(tmpdir):
    tmpdir.chdir()
    from nipype import config
    from nipype.interfaces.base import CommandLine
    provenance_state = config.get('execution', 'write_provenance')
    hash_state = config.get('execution', 'hash_method')
    config.enable_provenance()
    CommandLine('echo hello').run()
    config.set('execution', 'write_provenance', provenance_state)
    config.set('execution', 'hash_method', hash_state)
    assert tmpdir.join('provenance.provn').check()


def test_safe_encode():
    a = '\xc3\xa9lg'
    out = safe_encode(a)
    assert out.value == a



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ...utils.filemanip import split_filename
from ..base import (traits, TraitedSpec, File, StdOutCommandLine,
                    StdOutCommandLineInputSpec)


class SFPICOCalibDataInputSpec(StdOutCommandLineInputSpec):
    snr = traits.Float(
        argstr='-snr %f',
        units='NA',
        desc=('Specifies  the  signal-to-noise ratio of the '
              'non-diffusion-weighted measurements to use in simulations.'))
    scheme_file = File(
        exists=True,
        argstr='-schemefile %s',
        mandatory=True,
        desc='Specifies the scheme file for the diffusion MRI data')
    info_file = File(
        desc='The name to be given to the information output filename.',
        argstr='-infooutputfile %s',
        mandatory=True,
        genfile=True,
        hash_files=False)  # Genfile and hash_files?
    trace = traits.Float(
        argstr='-trace %f',
        units='NA',
        desc='Trace of the diffusion tensor(s) used in the test function.')
    onedtfarange = traits.List(
        traits.Float,
        argstr='-onedtfarange %s',
        minlen=2,
        maxlen=2,
        units='NA',
        desc=('Minimum and maximum FA for the single tensor '
              'synthetic data.'))
    onedtfastep = traits.Float(
        argstr='-onedtfastep %f',
        units='NA',
        desc=('FA step size controlling how many steps there are '
              'between the minimum and maximum FA settings.'))
    twodtfarange = traits.List(
        traits.Float,
        argstr='-twodtfarange %s',
        minlen=2,
        maxlen=2,
        units='NA',
        desc=('Minimum and maximum FA for the two tensor '
              'synthetic data. FA is varied for both tensors '
              'to give all the different permutations.'))
    twodtfastep = traits.Float(
        argstr='-twodtfastep %f',
        units='NA',
        desc=('FA step size controlling how many steps there are '
              'between the minimum and maximum FA settings '
              'for the two tensor cases.'))
    twodtanglerange = traits.List(
        traits.Float,
        argstr='-twodtanglerange %s',
        minlen=2,
        maxlen=2,
        units='NA',
        desc=('Minimum and maximum crossing angles '
              'between the two fibres.'))
    twodtanglestep = traits.Float(
        argstr='-twodtanglestep %f',
        units='NA',
        desc=('Angle step size controlling how many steps there are '
              'between the minimum and maximum crossing angles for '
              'the two tensor cases.'))
    twodtmixmax = traits.Float(
        argstr='-twodtmixmax %f',
        units='NA',
        desc=
        ('Mixing parameter controlling the proportion of one fibre population '
         'to the other. The minimum mixing parameter is (1 - twodtmixmax).'))
    twodtmixstep = traits.Float(
        argstr='-twodtmixstep %f',
        units='NA',
        desc=('Mixing parameter step size for the two tensor cases. '
              'Specify how many mixing parameter increments to use.'))
    seed = traits.Float(
        argstr='-seed %f',
        units='NA',
        desc=
        'Specifies the random seed to use for noise generation in simulation trials.'
    )


class SFPICOCalibDataOutputSpec(TraitedSpec):
    PICOCalib = File(exists=True, desc='Calibration dataset')
    calib_info = File(exists=True, desc='Calibration dataset')


class SFPICOCalibData(StdOutCommandLine):
    """
    Generates Spherical Function PICo Calibration Data.

    SFPICOCalibData creates synthetic data for use with SFLUTGen. The
    synthetic data is generated using a mixture of gaussians, in the
    same way datasynth generates data.  Each voxel of data models a
    slightly different fibre configuration (varying FA and fibre-
    crossings) and undergoes a random rotation to help account for any
    directional bias in the chosen acquisition scheme.  A second file,
    which stores information about the datafile, is generated along with
    the datafile.

    Example 1
    ---------
    To create a calibration dataset using the default settings

    >>> import nipype.interfaces.camino as cam
    >>> calib = cam.SFPICOCalibData()
    >>> calib.inputs.scheme_file = 'A.scheme'
    >>> calib.inputs.snr = 20
    >>> calib.inputs.info_file = 'PICO_calib.info'
    >>> calib.run()           # doctest: +SKIP

    The default settings create a large dataset (249,231 voxels), of
    which 3401 voxels contain a single fibre population per voxel and
    the rest of the voxels contain two fibre-populations. The amount of
    data produced can be varied by specifying the ranges and steps of
    the parameters for both the one and two fibre datasets used.

    Example 2
    ---------
    To create a custom calibration dataset

    >>> import nipype.interfaces.camino as cam
    >>> calib = cam.SFPICOCalibData()
    >>> calib.inputs.scheme_file = 'A.scheme'
    >>> calib.inputs.snr = 20
    >>> calib.inputs.info_file = 'PICO_calib.info'
    >>> calib.inputs.twodtfarange = [0.3, 0.9]
    >>> calib.inputs.twodtfastep = 0.02
    >>> calib.inputs.twodtanglerange = [0, 0.785]
    >>> calib.inputs.twodtanglestep = 0.03925
    >>> calib.inputs.twodtmixmax = 0.8
    >>> calib.inputs.twodtmixstep = 0.1
    >>> calib.run()              # doctest: +SKIP

    This would provide 76,313 voxels of synthetic data, where 3401 voxels
    simulate the one fibre cases and 72,912 voxels simulate the various
    two fibre cases. However, care should be taken to ensure that enough
    data is generated for calculating the LUT.      # doctest: +SKIP
    """
    _cmd = 'sfpicocalibdata'
    input_spec = SFPICOCalibDataInputSpec
    output_spec = SFPICOCalibDataOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['PICOCalib'] = os.path.abspath(self._gen_outfilename())
        outputs['calib_info'] = os.path.abspath(self.inputs.info_file)
        return outputs

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.scheme_file)
        return name + '_PICOCalib.Bfloat'


class SFLUTGenInputSpec(StdOutCommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='-inputfile %s',
        mandatory=True,
        desc='Voxel-order data of the spherical functions peaks.')
    info_file = File(
        argstr='-infofile %s',
        mandatory=True,
        desc=('The Info file that corresponds to the calibration '
              'datafile used in the reconstruction.'))
    outputstem = traits.Str(
        'LUT',
        argstr='-outputstem %s',
        desc=
        ('Define the name of the generated luts.  The form of the filenames will be '
         '[outputstem]_oneFibreSurfaceCoeffs.Bdouble and '
         '[outputstem]_twoFibreSurfaceCoeffs.Bdouble'),
        usedefault=True)
    pdf = traits.Enum(
        'bingham',
        'watson',
        argstr='-pdf %s',
        desc=
        ('Sets the distribution to use for the calibration. The default is the Bingham '
         'distribution, which allows elliptical  probability  density  contours. '
         'Currently supported options are: '
         '  bingham - The Bingham distribution, which allows elliptical probability '
         '            density contours. '
         '  watson - The Watson distribution. This distribution is rotationally symmetric.'
         ),
        usedefault=True)
    binincsize = traits.Int(
        argstr='-binincsize %d',
        units='NA',
        desc=
        ('Sets the size of the bins.  In the case of 2D histograms such as the '
         'Bingham, the bins are always square. Default is 1.'))
    minvectsperbin = traits.Int(
        argstr='-minvectsperbin %d',
        units='NA',
        desc=
        ('Specifies the minimum number of fibre-orientation estimates a bin '
         'must contain before it is used in the lut line/surface generation. '
         'Default is 50. If you get the error "no fibre-orientation estimates '
         'in histogram!", the calibration data set is too small to get enough '
         'samples in any of the  histogram  bins. You can decrease the minimum '
         'number  per  bin to get things running in quick tests, but the sta- '
         'tistics will not be reliable and for serious applications, you need  '
         'to increase the size of the calibration data set until the error goes.'
         ))
    directmap = traits.Bool(
        argstr='-directmap',
        desc=
        ('Use direct mapping between the eigenvalues and the distribution parameters '
         'instead of the log of the eigenvalues.'))
    order = traits.Int(
        argstr='-order %d',
        units='NA',
        desc=
        ('The order of the polynomial fitting the surface. Order 1 is linear. '
         'Order 2 (default) is quadratic.'))


class SFLUTGenOutputSpec(TraitedSpec):
    lut_one_fibre = File(exists=True, desc='PICo lut for one-fibre model')
    lut_two_fibres = File(exists=True, desc='PICo lut for two-fibre model')


class SFLUTGen(StdOutCommandLine):
    """
    Generates PICo lookup tables (LUT) for multi-fibre methods such as
    PASMRI and Q-Ball.

    SFLUTGen creates the lookup tables for the generalized multi-fibre
    implementation of the PICo tractography algorithm.  The outputs of
    this utility are either surface or line coefficients up to a given
    order. The calibration can be performed for different distributions,
    such as the Bingham and Watson distributions.

    This utility uses calibration data generated from SFPICOCalibData
    and peak information created by SFPeaks.

    The utility outputs two lut's, *_oneFibreSurfaceCoeffs.Bdouble and
    *_twoFibreSurfaceCoeffs.Bdouble. Each of these files contains big-
    endian doubles as standard. The format of the output is: ::

          dimensions    (1 for Watson, 2 for Bingham)
          order         (the order of the polynomial)
          coefficient_1
          coefficient_2
          ...
          coefficient_N

    In  the case of the Watson, there is a single set of coefficients,
    which are ordered: ::

          constant, x, x^2, ..., x^order.

    In the case of the Bingham, there are two sets of coefficients (one
    for each surface), ordered so that: ::

          for j = 1 to order
            for k = 1 to order
              coeff_i = x^j * y^k
          where j+k < order

    Example
    ---------
    To create a calibration dataset using the default settings

    >>> import nipype.interfaces.camino as cam
    >>> lutgen = cam.SFLUTGen()
    >>> lutgen.inputs.in_file = 'QSH_peaks.Bdouble'
    >>> lutgen.inputs.info_file = 'PICO_calib.info'
    >>> lutgen.run()        # doctest: +SKIP
    """
    _cmd = 'sflutgen'
    input_spec = SFLUTGenInputSpec
    output_spec = SFLUTGenOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs[
            'lut_one_fibre'] = self.inputs.outputstem + '_oneFibreSurfaceCoeffs.Bdouble'
        outputs[
            'lut_two_fibres'] = self.inputs.outputstem + '_twoFibreSurfaceCoeffs.Bdouble'
        return outputs

    def _gen_outfilename(self):
        return '/dev/null'
